@@ -329,7 +329,8 @@ bool ConvertToTableIfPossible(const std::string &name,SEXP x,DTTable &returnTabl
         return true;
     }
     
-    if (Rf_isFrame(x)) {
+    // isDataFrame(x) instead? - https://cran.r-project.org/doc/manuals/r-devel/R-exts.html
+    if (Rf_isDataFrame(x)) {
         returnTable = ConvertToTable(DataFrame(x));
         return true;
     }
@@ -383,7 +384,7 @@ DTTableColumn ConvertToColumn(const std::string &name,SEXP x)
         return DTTableColumn::NumberColumn(name,DTDoubleArray());
     } else {
         if (typeOfX==19) {
-            if (Rf_isFrame(x)) {
+            if (Rf_isDataFrame(x)) {
                 DTTable subTable = ConvertToTable(DataFrame(x));
                 return DTTableColumn::TableColumn(name,subTable);
                 // Rcout << name << " is a frame" << endl;
